using System;
using System.Drawing;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using RPGOnline.Properties;

namespace RPGOnline.Controls.Map.Layers
{
    public class BackgroundLayer : ILayer
    {
        private readonly Device m_device;
        private readonly Map m_map;
        private VertexBuffer m_backGround;
        private Texture m_backGroundTexture;

        public BackgroundLayer(Device device, Map map)
        {
            m_device = device;
            m_map = map;

            m_backGround = new VertexBuffer(typeof(CustomVertex.TransformedColoredTextured), 4, m_device, Usage.WriteOnly, CustomVertex.TransformedColoredTextured.Format, Pool.Managed);
            var vertices = new CustomVertex.TransformedColoredTextured[4];

            int width = m_map.MapSize.Width*m_map.TileSize.Width;
            int height = m_map.MapSize.Height*m_map.TileSize.Height;

            vertices[0].Position = new Vector4(0, 0, 0, 1.0f);
            vertices[0].Color = Color.White.ToArgb();
            vertices[0].Tu = 0;
            vertices[0].Tv = 0;

            vertices[1].Position = new Vector4(width, 0, 0, 1.0f);
            vertices[1].Color = Color.White.ToArgb();
            vertices[1].Tu = m_map.MapSize.Width * 2;
            vertices[1].Tv = 0;

            vertices[2].Position = new Vector4(0, height, 0, 1.0f);
            vertices[2].Color = Color.White.ToArgb();
            vertices[2].Tu = 0;
            vertices[2].Tv = m_map.MapSize.Height * 2;

            vertices[3].Position = new Vector4(width, height, 0, 1.0f);
            vertices[3].Color = Color.White.ToArgb();
            vertices[3].Tu = m_map.MapSize.Width * 2;
            vertices[3].Tv = m_map.MapSize.Height * 2;

            m_backGround.SetData(vertices, 0, LockFlags.Discard);
            m_backGroundTexture = new Texture(m_device, Resources.Background, Usage.None, Pool.Managed);
        }

        public void Clear()
        {
        }

        public void Draw()
        {
            m_device.SetTexture(0, m_backGroundTexture);
            m_device.VertexFormat = m_backGround.Description.VertexFormat;
            m_device.SetStreamSource(0, m_backGround, 0);

            m_device.DrawPrimitives(PrimitiveType.TriangleStrip, 0, 2);

            m_device.SetTexture(0, null);
        }

        public MapCell this[int x, int y]
        {
            get { throw new NotImplementedException(); }
        }
    }
}