using System;
using System.Drawing;
using System.Runtime.InteropServices;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace RPGOnline.Controls.Map.Layers
{
    public class GridLayer : ILayer
    {
        private readonly Device m_device;
        private readonly Map m_map;
        private VertexBuffer m_grid;

        public GridLayer(Device device, Map map)
        {
            m_device = device;
            m_map = map;

            int verticalLines = m_map.MapSize.Width + 1;
            int horizontalLines = m_map.MapSize.Height + 1;

            m_grid = new VertexBuffer(typeof(CustomVertex.TransformedColored), 2 * verticalLines + 2 * horizontalLines, m_device, Usage.WriteOnly, CustomVertex.TransformedColored.Format, Pool.Managed);
            var vertices = new CustomVertex.TransformedColored[2*verticalLines + 2*horizontalLines];

            int index = 0;
            for (int x = 0; x <= m_map.MapSize.Width; x++)
            {
                vertices[index].Position = new Vector4(x*m_map.TileSize.Width, 0, 0, 1.0f);
                vertices[index++].Color = Color.LightGray.ToArgb();

                vertices[index].Position = new Vector4(x*m_map.TileSize.Width,
                                                       m_map.MapSize.Height*m_map.TileSize.Height, 0, 1.0f);
                vertices[index++].Color = Color.LightGray.ToArgb();
            }

            for (int y = 0; y <= m_map.MapSize.Height; y++)
            {
                vertices[index].Position = new Vector4(0, y*m_map.TileSize.Height, 0, 1.0f);
                vertices[index++].Color = Color.LightGray.ToArgb();

                vertices[index].Position = new Vector4(m_map.MapSize.Width*m_map.TileSize.Width, y*m_map.TileSize.Height,
                                                       0, 1.0f);
                vertices[index++].Color = Color.LightGray.ToArgb();
            }

            m_grid.SetData(vertices, 0, LockFlags.Discard);
        }

        public bool Visible { get; set; }

        public void Clear()
        {
        }

        public void Draw()
        {
            if (!Visible)
                return;

            int linesCount = (m_map.MapSize.Width + 1) + (m_map.MapSize.Height + 1);

            m_device.Transform.World = Matrix.Identity;
            m_device.VertexFormat = CustomVertex.TransformedColored.Format;
            m_device.SetStreamSource(0, m_grid, 0);
            m_device.DrawPrimitives(PrimitiveType.LineList, 0, linesCount);
        }

        public MapCell this[int x, int y]
        {
            get { throw new NotImplementedException(); }
        }
    }
}