using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Threading.Tasks;
using RPGOnline.Controls.Map.Layers.Lighting;
using RPGOnline.Extensions;
using RPGOnline.Properties;

namespace RPGOnline.Controls.Map.Layers
{
    public class LightingLayer : ILayer
    {
        private readonly Map m_map;
        private LightsCollection m_lights;
        private Bitmap m_tempImage;
        private int[,] m_mapStates;
        private bool m_showObstructions;
        private SolidBrush m_obstructionsBrush;
        private SolidBrush m_clearBrush;
        private bool m_showLights;
        private Image m_lightImage;

        public LightingLayer(Map map)
        {
            m_lightImage = Resources.Light;
            m_obstructionsBrush = new SolidBrush(Color.FromArgb(128, Color.Red));
            m_clearBrush = new SolidBrush(Color.FromArgb(200, Color.Black));
            m_map = map;
            m_tempImage = new Bitmap(m_map.MapSize.Width*m_map.TileSize.Width,
                                     m_map.MapSize.Height*m_map.TileSize.Height,
                                     PixelFormat.Format32bppArgb);
            m_mapStates = new int[m_map.MapSize.Width, m_map.MapSize.Height];
            m_lights = new LightsCollection(m_map, this);
        }

        public LightsCollection Lights
        {
            get { return m_lights; }
            set { m_lights = value; }
        }

        public void Clear()
        {
            using(Graphics g = Graphics.FromImage(m_tempImage))
            {
                GraphicsUnit graphicsUnit = GraphicsUnit.Pixel;
                g.FillRectangle(m_clearBrush, m_tempImage.GetBounds(ref graphicsUnit));
            }
        }

        public void Draw()
        {
            Clear();

            using (Graphics g = Graphics.FromImage(m_tempImage))
            {
                m_lights.ForEach<Light>(l => l.Draw(g));
            }

            /*GraphicsUnit graphicsUnit = GraphicsUnit.Pixel;
            Rectangle bounds = Rectangle.Truncate(m_tempImage.GetBounds(ref graphicsUnit));
            
            BitmapData dstBD = bitmap.LockBits(bounds, ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
            BitmapData srcBD = m_tempImage.LockBits(bounds, ImageLockMode.ReadOnly, PixelFormat.Format24bppRgb);

            IntPtr dstScan0 = dstBD.Scan0;
            IntPtr srcScan0 = srcBD.Scan0;

            unsafe
            {
                byte* pDst = (byte*)(void*)dstScan0;
                byte* pSrc = (byte*)(void*)srcScan0;

                int totalBytes = bitmap.Width*3*bitmap.Height;
                byte* pSrcEnd = pSrc + totalBytes;
                while (pSrcEnd != pSrc)
                {
                    *pDst = (byte) ((*pDst**pSrc)/255);

                    pDst++;
                    pSrc++;
                }
            }

            m_tempImage.UnlockBits(srcBD);
            bitmap.UnlockBits(dstBD);

            if (m_showObstructions)
            {
                using (Graphics g = Graphics.FromImage(bitmap))
                {
                    for (int x = 0; x < m_map.MapSize.Width; x++)
                        for (int y = 0; y < m_map.MapSize.Height; y++)
                            if (m_mapStates[x, y] != 0)
                                g.FillRectangle(m_obstructionsBrush, x*m_map.TileSize.Width, y*m_map.TileSize.Height,
                                                m_map.TileSize.Width, m_map.TileSize.Height);
                }
            }

            if(m_showLights)
            {
                using(Graphics g = Graphics.FromImage(bitmap))
                {
                    foreach (Light light in m_lights)
                    {
                        Rectangle lightSprite = new Rectangle(light.Position, m_lightImage.Size);
                        g.DrawImage(m_lightImage, lightSprite);
                    }
                }
            }*/
        }

        public MapCell this[int x, int y]
        {
            get { throw new NotImplementedException(); }
        }

        public bool CheckPoint(int x, int y)
        {
            //check coords are legal
            if (x >= 0 & x < m_map.MapSize.Width & y >= 0 & y < m_map.MapSize.Height)
            {
                //if the cell is empty return true
                if (m_mapStates[x, y] == 0)
                    return true;
            }
            //not legal coords
            return false;
        }

        public void SetObstruction(int x, int y)
        {
            m_mapStates[x, y] = 1;
            m_lights.ForEach<Light>(l => l.Invalidate());
        }

        public void ShowObstructions(bool enabled)
        {
            m_showObstructions = enabled;
        }

        public void ShowLights(bool enabled)
        {
            m_showLights = enabled;
        }
    }
}