using System.Drawing;
using System.Drawing.Imaging;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace RPGOnline.Controls.Map.Layers
{
    public class MapLayer : ILayer
    {
        protected readonly Device m_device;
        private readonly Map m_map;

        public MapCell[,] Cells { get; private set; }
        public string Name { get; set; }

        public MapLayer(Device device, string name, Map map)
        {
            Name = name;
            m_device = device;
            m_map = map;
            
            Clear();
        }

        

        public void Clear()
        {
            Cells = new MapCell[m_map.MapSize.Width, m_map.MapSize.Height];

            for (int x = 0; x < m_map.MapSize.Width; x++)
            {
                for (int y = 0; y < m_map.MapSize.Height; y++)
                {
                    Cells[x, y] = new MapCell();
                }
            }
        }

        public MapCell this[int x, int y]
        {
            get { return Cells[x, y]; }
        }

        public void Draw()
        {
            for (int x = 0; x < m_map.MapSize.Width; x++)
            {
                for (int y = 0; y < m_map.MapSize.Height; y++)
                {
                    MapCell mapCell = Cells[x, y];

                    if (mapCell.Frames == null)
                        continue;

                    DrawTile(mapCell, x, y);
                }
            }

            m_device.SetTexture(0, null);
        }

        protected virtual void DrawTile(MapCell tile, int x, int y)
        {
            if (tile.Frames.Count == 0)
                return;

            tile.Frame.Draw(m_device, x, y);
        }

        public override string ToString()
        {
            return Name;
        }
    }
}