using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Imaging;
using Microsoft.DirectX.Direct3D;
using RPGOnline.Controls.Map.Layers;
using RPGOnline.Properties;
using Light = RPGOnline.Controls.Map.Layers.Lighting.Light;
using LightsCollection = RPGOnline.Controls.Map.Layers.Lighting.LightsCollection;

namespace RPGOnline.Controls.Map
{
    public class Map
    {
        private readonly Device m_device;

        [Category("Map")]
        public Size TileSize { get; private set; }
        [Category("Map")]
        public Size MapSize { get; private set; }
        [Browsable(false)]
        public List<ILayer> Layers { get { return m_layers; } }
        [Browsable(false)]
        public List<MapTileset> TileSets { get { return m_tileSets; } }
        [Browsable(false)]
        public ILayer CurrentLayer { get; set; }
        public LightsCollection Lights
        {
            get { return LightingLayer.Lights; }
            set { LightingLayer.Lights = value; }
        }

        private List<ILayer> m_layers = new List<ILayer>();
        private List<MapTileset> m_tileSets = new List<MapTileset>();
        [Browsable(false)]
        public LightingLayer LightingLayer { get; private set; }
        [Browsable(false)]
        public OverlayLayer OverlayLayer { get; private set; }

    	public Map(Device device, Size tileSize, Size mapSize)
        {
    	    m_device = device;
    	    TileSize = tileSize;
            MapSize = mapSize;

            NewLayer("Terrain");
            CurrentLayer = m_layers[0];
            LightingLayer = new LightingLayer(this);
            OverlayLayer = new OverlayLayer(m_device, this);
        }

        public MapLayer NewLayer(string name)
        {
            MapLayer mapLayer = new MapLayer(m_device, name, this);
            m_layers.Add(mapLayer);
            return mapLayer;
        }

        public void Draw()
        {
            m_layers.ForEach(l => l.Draw());
            LightingLayer.Draw();
            OverlayLayer.Draw();
        }

        public void GetCellCoordsAt(Point position, out Point cellCoords)
        {
            cellCoords = new Point(
                position.X / TileSize.Width,
                position.Y / TileSize.Height
            );
        }

        public void AddLight(Color color, Point point, float radius)
        {
            LightingLayer.Lights.Add(new Light(this, LightingLayer)
            {
                Color = color, Position = point, Radius = radius
            });
        }
    }
}